@@ -2,28 +2,39 @@
 
 #### ctrLoadQueryIntoDb ####
 
+# test
+expect_error(
+  suppressWarnings(
+    suppressMessages(
+      ctrLoadQueryIntoDb(
+        queryterm = "query=",
+        register = "EUCTR",
+        con = dbc))),
+  "more than 10,000) trials")
+
+# next
 q <- paste0("https://www.clinicaltrialsregister.eu/ctr-search/search?query=",
             "neuroblastoma&status=completed&phase=phase-one&country=pl")
 
 # test
 expect_message(
-  tmp_test <- suppressWarnings(
+  tmpTest <- suppressWarnings(
     ctrLoadQueryIntoDb(
       queryterm = q,
       con = dbc)),
   "Imported or updated")
 
 # test
-expect_true(tmp_test$n > 10L)
+expect_true(tmpTest$n > 10L)
 
 # test
 expect_true(all(
   c("2007-000371-42-FR", "2010-019340-40-GB",
     "2010-019340-40-3RD") %in%
-    tmp_test$success))
+    tmpTest$success))
 
 # test
-expect_true(length(tmp_test$failed) == 0L)
+expect_true(length(tmpTest$failed) == 0L)
 
 
 #### ctrLoadQueryIntoDb update ####
@@ -35,7 +46,7 @@ hist <- suppressWarnings(dbQueryHistory(con = dbc))
 hist[nrow(hist), "query-term"] <-
   sub("query=", "", hist[nrow(hist), "query-term"])
 hist[nrow(hist), "query-timestamp"] <- "2000-01-01 00:00:00"
-  #
+#
 # convert into json object
 json <- jsonlite::toJSON(list("queries" = hist))
 #
@@ -60,19 +71,19 @@ expect_message(
 # query based on date is used since this avoids no trials are found
 
 date.today <- Sys.time()
-date.from  <- format(date.today - (60 * 60 * 24 * 15), "%Y-%m-%d")
-date.to    <- format(date.today - (60 * 60 * 24 *  9), "%Y-%m-%d")
+date.from  <- format(date.today - (60 * 60 * 24 * 9), "%Y-%m-%d")
+date.to    <- format(date.today - (60 * 60 * 24 * 4), "%Y-%m-%d")
 
-q <- paste0("https://www.clinicaltrialsregister.eu/ctr-search/search?query=",
+q <- paste0("https://www.clinicaltrialsregister.eu/ctr-search/search?query=cancer&phase=phase-two",
             "&dateFrom=", date.from, "&dateTo=", date.to)
 
 # test
 expect_message(
-  tmp_test <- suppressWarnings(
+  tmpTest <- suppressWarnings(
     ctrLoadQueryIntoDb(
-      paste0(q),
+      queryterm = paste0(q),
       con = dbc)),
-  "Imported or updated ")
+  "(Imported or updated|First result page empty)")
 
 # manipulate history to test updating
 hist <- suppressWarnings(dbQueryHistory(con = dbc))
@@ -97,24 +108,22 @@ nodbi::docdb_update(
 
 # test
 expect_message(
-  tmp_test <- suppressWarnings(
+  tmpTest <- suppressWarnings(
     ctrLoadQueryIntoDb(
       querytoupdate = "last",
+      verbose = TRUE,
       con = dbc)),
   "(Imported or updated|First result page empty)")
 
 # test
-expect_true(tmp_test$n > 10L)
+expect_true(tmpTest$n > 5L)
 
 # test
-expect_true(length(tmp_test$success) > 10L)
+expect_true(length(tmpTest$success) > 5L)
 
 
 #### ctrLoadQueryIntoDb results ####
 
-# https://www.clinicaltrialsregister.eu/ctr-search/search?query=&
-# age=under-18&phase=phase-three&resultsstatus=trials-with-results
-
 # get trials with results
 q <- paste0("https://www.clinicaltrialsregister.eu/ctr-search/search?query=",
             "2013-003420-37+OR+2009-011454-17+OR+2006-005357-29")
@@ -130,29 +139,33 @@ expect_message(
   "Imported or updated results for")
 
 # get results
-# dbFindFields(namepart = c("statistic"),
-#              con = dbc)
-result <- suppressWarnings(
-  dbGetFieldsIntoDf(
-    fields = c(
-      "a2_eudract_number",
-      "trialInformation.globalEndOfTrialDate",
-      "p_date_of_the_global_end_of_the_trial",
-      "trialInformation.recruitmentStartDate",
-      "x6_date_on_which_this_record_was_first_entered_in_the_eudract_database",
-      "subjectDisposition.postAssignmentPeriods.postAssignmentPeriod.arms.arm",
-      "endPoints.endPoint", # this is also a list
-      "trialInformation.analysisForPrimaryCompletion",
-      "e71_human_pharmacology_phase_i"
-    ),
-    con = dbc,
-    stopifnodata = FALSE)
+result <- suppressMessages(
+  suppressWarnings(
+    dbGetFieldsIntoDf(
+      fields = c(
+        "a2_eudract_number",
+        "trialInformation.globalEndOfTrialDate",
+        "p_date_of_the_global_end_of_the_trial",
+        "trialInformation.recruitmentStartDate",
+        "x6_date_on_which_this_record_was_first_entered_in_the_eudract_database",
+        "subjectDisposition.postAssignmentPeriods.postAssignmentPeriod.arms.arm",
+        "endPoints.endPoint",
+        "trialInformation.analysisForPrimaryCompletion",
+        "e71_human_pharmacology_phase_i"
+      ),
+      con = dbc,
+      stopifnodata = FALSE)
+  ))
+
+# test
+expect_true(
+  nrow(result) > 25L
 )
 
 # keep only one record for trial
 result <- suppressWarnings(suppressMessages(
-  result[ result[["_id"]] %in%
-            dbFindIdsUniqueTrials(con = dbc), ]
+  result[result[["_id"]] %in%
+           dbFindIdsUniqueTrials(con = dbc), ]
 ))
 
 # test
@@ -163,21 +176,30 @@ expect_true(all(as.Date(c("2013-10-28", "2018-03-13")) %in%
 expect_true("logical" == class(result[[
   "e71_human_pharmacology_phase_i"]]))
 
+# test
+expect_true("logical" == class(result[[
+  "trialInformation.analysisForPrimaryCompletion"]]))
+
 # test
 expect_true("Date" == class(result[[
   "trialInformation.globalEndOfTrialDate"]]))
 
+# test
+expect_true("list" == class(result[[
+  "endPoints.endPoint"]]))
+
 # test
 expect_true(
   sum(nchar(
     # note: function
     # is deprecated
-    dfListExtractKey(
-      df = result,
-      list.key = list(
-        c("endPoints.endPoint", "title"))
-    )[["value"]]
-  ), na.rm = TRUE)
+    suppressWarnings(
+      dfListExtractKey(
+        df = result,
+        list.key = list(
+          c("endPoints.endPoint", "title"))
+      )[["value"]]
+    )), na.rm = TRUE)
   > 3000L)
 
 # convert to long
@@ -195,7 +217,13 @@ expect_identical(
 
 # test
 expect_true(
-  nrow(df) > 3000L
+  nrow(df) > 2500L
+)
+
+# extract
+df2 <- dfName2Value(
+  df = df,
+  valuename = "subjectDisposition.*postAssignmentPeriod.arms.arm.type.value"
 )
 
 # extract
@@ -227,7 +255,7 @@ df2 <- dfName2Value(
   df = df,
   valuename = "^endPoints.endPoint.statisticalAnalyses.statisticalAnalysis.statisticalHypothesisTest.value$",
   wherename = "endPoints.endPoint.statisticalAnalyses.statisticalAnalysis.statisticalHypothesisTest.method.value",
-  wherevalue = "HYPOTHESIS_METHOD"
+  wherevalue = "HYPOTHESIS_METHOD.*"
 )
 
 # test
@@ -266,12 +294,12 @@ expect_error(
   "Empty 'namepart' parameter.")
 
 # test
-tmp_test <- suppressMessages(suppressWarnings(
+tmpTest <- suppressMessages(suppressWarnings(
   dbFindFields(
     namepart = "date",
     con = dbc)))
-expect_true("character" %in% class(tmp_test))
-expect_true(length(tmp_test) >= 4L)
+expect_true("character" %in% class(tmpTest))
+expect_true(length(tmpTest) >= 4L)
 
 
 #### annotations #####
@@ -280,13 +308,13 @@ expect_true(length(tmp_test) >= 4L)
 expect_message(
   suppressWarnings(
     ctrLoadQueryIntoDb(
-      queryterm = "NCT01516567",
+      queryterm = "NCT00001209+OR+NCT00001436+OR+NCT00187109+OR+NCT01516567+OR+NCT01471782",
       register = "CTGOV",
       con = dbc,
       verbose = TRUE,
       annotation.text = "ANNO",
       annotation.mode = "replace")),
-  "Imported or updated 1 trial")
+  "Imported or updated 5 trial")
 
 # test
 expect_message(
@@ -310,31 +338,57 @@ expect_message(
       annotation.mode = "prepend")),
   "Imported or updated")
 
-tmp_test <- suppressWarnings(
-  dbGetFieldsIntoDf(
-    fields = "annotation",
-    con = dbc))
+tmpTest <- suppressMessages(
+  suppressWarnings(
+    dbGetFieldsIntoDf(
+      fields = "annotation",
+      con = dbc)))
 
-tmp_test <-
-  tmp_test[
-    tmp_test[["_id"]] %in%
+tmpTest <-
+  tmpTest[
+    tmpTest[["_id"]] %in%
       suppressMessages(
         suppressWarnings(
           dbFindIdsUniqueTrials(
-            con = dbc))) , ]
+            con = dbc))), ]
 
 # test
-expect_equal(sort(tmp_test[["annotation"]]),
-             sort(c("EU ANNO", "ANNO")))
+tmpTest <- tmpTest[tmpTest[["annotation"]] != "", ]
+expect_equal(sort(unique(tmpTest[["annotation"]])),
+             sort(unique(c("EU ANNO", "ANNO"))))
 
-# test
-expect_message(
+#### deduplicate ####
+
+trialsCtgov <- suppressMessages(
   suppressWarnings(
     dbFindIdsUniqueTrials(
       con = dbc,
-      preferregister = "CTGOV")),
-  "Concatenating 1 records from CTGOV and [2-9][0-9] from EUCTR")
+      preferregister = "CTGOV")))
 
+trialsEuctr <- suppressMessages(
+  suppressWarnings(
+    dbFindIdsUniqueTrials(
+      con = dbc,
+      preferregister = "EUCTR")))
+
+# test
+expect_equal(length(trialsCtgov), length(trialsEuctr))
+
+# test
+expect_true(
+  all(c("NCT00001209", "NCT00001436", "NCT00187109", "NCT01516567") %in%
+        trialsCtgov))
+# test
+expect_false(
+  any(c("2010-024264-18-GB") %in% trialsCtgov))
+
+# test
+expect_true(
+  all(c("NCT00001209", "NCT00001436", "NCT00187109", "NCT01516567") %in%
+        trialsEuctr))
+# test
+expect_false(
+  any(c("NCT01471782") %in% trialsEuctr))
 
 #### dbFindIdsUniqueTrials #####
 
@@ -344,7 +398,23 @@ expect_message(
     dbFindIdsUniqueTrials(
       con = dbc,
       preferregister = "EUCTR")),
-  "Searching for duplicates")
+  "Searching for duplicate trials")
+
+# test
+expect_error(
+  dbFindIdsUniqueTrials(
+    con = dbc,
+    preferregister = "WRONG"),
+  "Register not known")
+
+# test
+expect_error(
+  suppressMessages(
+    suppressWarnings(
+      dbFindIdsUniqueTrials(
+        con = dbc,
+        prefermemberstate = "WRONG"))),
+  "prefermemberstate does not match")
 
 # test
 expect_message(
@@ -352,34 +422,43 @@ expect_message(
     dbFindIdsUniqueTrials(
       con = dbc,
       preferregister = "CTGOV")),
-  "Returning keys \\(_id\\) of [2-9][0-9]")
+  "Returning keys \\(_id\\) of [1-9][0-9]+")
 
 # test
 expect_warning(
   suppressMessages(
-    tmp_test <- dbFindIdsUniqueTrials(
+    tmpTest <- dbFindIdsUniqueTrials(
       con = dbc,
       prefermemberstate = "3RD",
       include3rdcountrytrials = FALSE)),
   "Preferred EUCTR version set to 3RD country trials")
 
 # test, reusing the query string
-tmp_q <- strsplit(q, "+OR+", fixed = TRUE)[[1]]
-tmp_q <- gsub(".+=(.?)", "\\1", tmp_q)
+tmpQ <- strsplit(q, "+OR+", fixed = TRUE)[[1]]
+tmpQ <- gsub(".+=(.?)", "\\1", tmpQ)
 expect_true(all(
-  tmp_q %in%
-    gsub("([0-9]{4}-[0-9]{6}-[0-9]{2})-.*", "\\1", tmp_test)))
-
+  tmpQ %in%
+    gsub("([0-9]{4}-[0-9]{6}-[0-9]{2})-.*", "\\1", tmpTest)))
 
 #### dbGetFieldsIntoDf ####
 
+# test
+expect_error(
+  dbGetFieldsIntoDf(
+    fields = 1:3,
+    con = dbc),
+  "Input should be a vector of strings of field names")
+
 # test
 expect_error(
   suppressWarnings(
-    dbGetFieldsIntoDf(
-      fields = c(NA, "willNeverBeFound"),
-      con = dbc)),
-  "For field 'willNeverBeFound' no data could be extracted")
+    suppressMessages(
+      dbGetFieldsIntoDf(
+        fields = c(NA, "willNeverBeFound"),
+        con = dbc))),
+  paste0(
+    "No data could be extracted for",
+    "|No records with values for any specified field"))
 
 # test
 expect_error(
@@ -397,8 +476,6 @@ tmpf <- suppressMessages(
     dbFindFields(
       namepart = ".*",
       con = dbc)))
-# remove empty field names
-tmpf <- tmpf[tmpf != ""]
 # get all data (takes long with sqlite)
 result <- suppressMessages(
   suppressWarnings(
@@ -414,13 +491,28 @@ tmpc <- sapply(result, class,
 tmpc <- unlist(tmpc)
 tmpc <- table(tmpc)
 
+# src_mongo:
+# tmpc
+# character      Date   integer      list   logical
+#       558        10        19         3        79
+
+# src_mongo:
+# tmpc
+# character Date   integer      list   logical
+# 561         10        19         3        59
+
 # tests
-expect_true(tmpc[["character"]] > 60)
+expect_true(tmpc[["character"]] > 50)
 expect_true(tmpc[["Date"]]      >  5)
-expect_true(tmpc[["integer"]]   > 10)
-expect_true(tmpc[["list"]]      > 10)
 expect_true(tmpc[["logical"]]   > 50)
 
+# not testing for lists, because
+# src_mongo returns only non-object
+# fields when searching for all fields
+# with dbFindFields(".*") whereas
+# src_sqlite returns names of both
+# objects and terminal / no more nested
+# fields
 
 #### ctrOpenSearchPagesInBrowser #####
 
@@ -430,4 +522,3 @@ expect_message(
     ctrOpenSearchPagesInBrowser(
       dbQueryHistory(con = dbc)[1, ])),
   "Opening browser for search:")
-
